<?php

/**
 * @file
 * Provide views data for debugger.module.
 */

/**
 * Implementation of hook_views_handlers().
 */
/*
function debugger_views_handlers() {
  return array(
    'handlers' => array(
      'debugger_views_handler_field_module' => array(
        'parent' => 'views_handler_field',
        'path' => drupal_get_path('module', 'debugger') . '/views/handlers',
      ),
      'debugger_views_handler_field_count' => array(
        'parent' => 'views_handler_field',
        'path' => drupal_get_path('module', 'debugger') . '/views/handlers',
      ),
      'debugger_views_handler_sort_count' => array(
        'parent' => 'views_handler_sort',
        'path' => drupal_get_path('module', 'debugger') . '/views/handlers',
      ),
      'debugger_views_handler_sort_date' => array(
        'parent' => 'views_handler_sort_date',
        'path' => drupal_get_path('module', 'debugger') . '/views/handlers',
      ),
      'debugger_views_handler_filter_in_operator_module' => array(
        'parent' => 'views_handler_filter_in_operator',
        'path' => drupal_get_path('module', 'debugger') . '/views/handlers',
      ),
    ),
  );
}
 */

/**
 * Implementation of hook_views_data().
 */
function debugger_views_data() {

  module_load_include('inc', 'debugger'); // include additional functions
  $data = array();

  /* REQUESTS */
  $data['debugger_requests']['table']['group'] = t('Debugger: Requests');
  $data['debugger_requests']['table']['base'] = array(
    'field' => 'rid',
    'title' => t('Debugger: Requests'),
    'help'  => t('Requests data generated by Drupal debugger.'),
  );
  $data['debugger_requests']['table']['join'] = array(
    'debugger_functions' => array(
      'field'      => 'fncid',
      'left_field' => 'fncid',
      'type' => 'inner',
    ),
    'users' => array(
      'field'      => 'uid',
      'left_field' => 'uid',
      'type' => 'inner',
    ),
    'fid' => array(
      'field'      => 'fid',
      'left_field' => 'fid',
      'type' => 'inner',
    ),
    /* TODO: rpath ? */
  );
  $data['debugger_requests']['rid'] = array(
    'title' => t('Request ID'),
    'help' => t('The unique id of a request'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by request id'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['path'] = array(
    'title' => t('Path'),
    'help'  => t('Path of the request'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
    'filter' => array(
      'help'    => t('Filter by path'),
      'handler' => 'views_handler_filter_in_operator',
      'options callback' => 'debugger_make_list',
      'options arguments' => array('path', 'requests'),
    ),
    'sort' => array(
      'help' => t('Sort by path'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['rpath'] = array(
    'title' => t('Router path'),
    'help' => t('Router path of the request'),
    'field' => array(
      'handler' => 'views_handler_field',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
    'filter' => array(
      'help'    => t('Filter by path'),
      'handler' => 'views_handler_filter_in_operator',
      'options callback' => 'debugger_make_list',
      'options arguments' => array('rpath', 'requests'),
    ),
    'sort' => array(
      'help' => t('Sort by router path'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['fncid'] = array(
    'title' => t('Function ID of the menu router callback'),
    'help' => t('The function id of the menu router callback.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Function ID of the menu router callback.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_functions',
      'relationship field' => 'fncid',
      'label' => t('Debugger: Function ID of the menu callback'),
    ),
  );
  $data['debugger_requests']['mem_fncid'] = array(
    'title' => t('Function ID which used the most memory'),
    'help' => t('The function id of the request which used the most memory.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Function ID which used the most memory.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_functions',
      'base field' => 'fncid',
      'relationship field' => 'mem_fncid',
      'label' => t('Debugger: Function ID which used the most memory'),
    ),
  );
  $data['debugger_requests']['slow_fncid'] = array(
    'title' => t('Function ID which was the slowest'),
    'help' => t('The function ID of the request which was the slowest.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Function ID which was the slowest.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_functions',
      'base field' => 'fncid',
      'relationship field' => 'slow_fncid',
      'label' => t('Debugger: Function ID (the slowest)'),
    ),
  );
  $data['debugger_requests']['slow_qid'] = array(
    'title' => t('Query ID which was the slowest'),
    'help' => t('The database query id of the request which was the slowest.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Query ID which was the slowest.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_db_queries',
      'base field' => 'qid',
      'relationship field' => 'slow_qid',
      'label' => t('Debugger: SQL query (the slowest)'),
    ),
  );
  $data['debugger_requests']['query'] = array(
    'title' => t('Query path'),
    'help'  => t('URI query path of the request'),
      'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'help' => t('Sort by path query'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['uid'] = array(
    'title' => t('User ID who sent the request.'),
    'help' => t('The user id of the user who sent the request.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by User ID who sent the request.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'users',
      'relationship field' => 'uid',
      'label' => t('Debugger: User ID'),
    ),
  );
  $data['debugger_requests']['time'] = array(
    'title' => t('Total time'),
    'help'  => t('The total time of that request'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
      'float' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by request time'),
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['memory'] = array(
    'title' => t('Memory used'),
    'help'  => t('The total memory used by the request'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by memory'),
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['num_sql'] = array(
    'title' => t('Number of SQL queries'),
    'help'  => t('The total number of SQL queries run on the request'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['num_ticks'] = array(
    'title' => t('Number of ticks'),
    'help'  => t('The total number of debugger ticks run on that request'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'help'    => t('Sort by number of ticks.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['num_err'] = array(
    'title' => t('Number of error messages'),
    'help'  => t('The total number of error messages of that request'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'help'    => t('Sort by number of error messages.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['num_warn'] = array(
    'title' => t('Number of warning messages'),
    'help'  => t('The total number of warning messages of that request'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'help'    => t('Sort by number of warning messages.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['num_notices'] = array(
    'title' => t('Number of notice messages'),
    'help'  => t('The total number of notice messages of that request'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'help'    => t('Sort by number of notice messages.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_requests']['timestamp'] = array(
    'title' => t('Timestamp'),
    'help'    => t('Date and time request was sent'),
    'field' => array(
      'handler' => 'views_handler_field_date',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by date and time of the requests'),
      'handler' => 'views_handler_filter_date',
    ),
    'sort' => array(
      'help'    => t('Sort by date and time of the requests'),
      'handler' => 'debugger_views_handler_sort_date',
    ),
  );
  $data['debugger_requests']['options'] = array(
    'title' => t('Options'),
    'help' => t('Additional options of that request.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => FALSE,
    ),
  );

  /* FUNCTIONS */
  $data['debugger_functions']['table']['group'] = t('Debugger: Functions');
  $data['debugger_functions']['table']['base'] = array(
    'field' => 'fncid',
    'title' => t('Debugger: Functions'),
    'help'  => t('Functions generated by Drupal debugger.'),
  );
  $data['debugger_functions']['fncid'] = array(
    'title' => t('Function ID'),
    'help' => t('The function ID.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Function ID.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_functions']['name'] = array(
    'title' => t('Function name'),
    'help'  => t('Name of the function.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by function name.'),
      'handler' => 'views_handler_filter_in_operator',
      'options callback' => 'debugger_function_list',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
    'sort' => array(
      'help' => t('Sort by path'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_functions']['fid'] = array(
    'title' => t('File ID'),
    'help' => t('The file ID where the function is defined.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by File ID'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_files',
      'relationship field' => 'fid',
      'label' => t('Debugger: File'),
    ),
  );
  $data['debugger_functions']['line'] = array(
    'title' => t('Line'),
    'help' => t('The line where the function is defined.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by file line'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_functions']['args'] = array( // FIXME: format of arguments could be change
    'title' => t('Arguments'),
    'help' => t('The arguments of the function.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by arguments'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_functions']['description'] = array(
    'title' => t('Description'),
    'help'  => t('Description of function'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'help' => t('Sort by description'),
      'handler' => 'views_handler_sort',
    ),
  );

  /* FILE */
  $data['debugger_files']['table']['group'] = t('Debugger: Files');
  $data['debugger_files']['table']['base'] = array(
    'field' => 'fid',
    'title' => t('Debugger: Files'),
    'help'  => t('Files generated by Drupal debugger.'),
  );
  $data['debugger_files']['fid'] = array(
    'title' => t('File ID'),
    'help' => t('The File ID.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by File ID.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_files']['filepath'] = array(
    'title' => t('File Path'),
    'help'  => t('Relative path of the file'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
    'sort' => array(
      'help' => t('Sort by path'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_files']['module'] = array(
    'title' => t('Module'),
    'help' => t('The module where the file appear.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by module name'),
      'handler' => 'views_handler_filter_in_operator',
      'options callback' => 'debugger_module_list',
      // 'options arguments' => array('module', 'files'),
    ),
    'sort' => array(
      'help' => t('Sort by module name of the file'),
      'handler' => 'views_handler_sort',
    ),
    'argument' => array(
      'help' => t('Filter by module of the file'),
      'handler' => 'views_handler_argument_string',
    ),
  );

  /* SQL QUERIES */
  $data['debugger_sql_queries']['table']['group'] = t('Debugger: SQL Queries');
  $data['debugger_sql_queries']['table']['base'] = array(
    'field' => 'qid',
    'title' => t('Debugger: SQL Queries'),
    'help'  => t('SQL queries generated by Drupal debugger.'),
  );
  $data['debugger_sql_queries']['qid'] = array(
    'title' => t('Query ID'),
    'help' => t('The Query ID.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Query ID.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_sql_queries']['fncid'] = array(
    'title' => t('Function ID'),
    'help' => t('The function id where this query was executed.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Function ID.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_functions',
      'relationship field' => 'fncid',
      'label' => t('Debugger: Function'),
    ),
  );
  $data['debugger_sql_queries']['time'] = array(
    'title' => t('Execution time'),
    'help'  => t('The total execution time of that SQL query.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
      'float' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by query time'),
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

  /* TRACES */
  $data['debugger_traces']['table']['group'] = t('Debugger: Traces');
  $data['debugger_traces']['table']['base'] = array(
    'field' => 'tid',
    'title' => t('Debugger: Traces'),
    'help'  => t('Traces generated by Drupal debugger.'),
  );
  $data['debugger_traces']['table']['join'] = array(
    'debugger_functions' => array(
      'field'      => 'fncid',
      'left_field' => 'fncid',
      'type' => 'inner',
    ),
    'debugger_files' => array(
      'field'      => 'fid',
      'left_field' => 'fid',
      'type' => 'inner',
    ),
    'debugger_requests' => array(
      'field'      => 'rid',
      'left_field' => 'rid',
      'type' => 'inner',
    ),
    /* TODO: rpath ? */
  );
  $data['debugger_traces']['tid'] = array(
    'title' => t('Trace ID'),
    'help' => t('The Trace ID.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Trace ID.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_traces']['ptid'] = array(
    'title' => t('Trace ID of parent'),
    'help' => t('The Trace ID of the parent.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Trace ID of parent.'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_traces']['fncid'] = array(
    'title' => t('Function ID'),
    'help' => t('The function id where this query was executed.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Function ID.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_functions',
      'relationship field' => 'fncid',
      'label' => t('Debugger: Function'),
    ),
  );
  $data['debugger_traces']['pfncid'] = array(
    'title' => t('Function ID of parent'),
    'help' => t('The function id which called the main function.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by Function ID of parent.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_functions',
      'base field' => 'fncid',
      'relationship field' => 'pfncid',
      'label' => t('Debugger: Function Parent'),
    ),
  );
  $data['debugger_traces']['fid'] = array(
    'title' => t('File ID'),
    'help' => t('The file id of the trace.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by File ID'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_files',
      'relationship field' => 'fid',
      'label' => t('Debugger: File'),
    ),
  );
  $data['debugger_traces']['line'] = array(
    'title' => t('Line'),
    'help' => t('The line where the function is defined.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by file line'),
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_traces']['rid'] = array(
    'title' => t('Request ID'),
    'help' => t('The unique id of a request associated with that trace.'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
    'sort' => array(
      'help' => t('Sort by request id of the trace.'),
      'handler' => 'views_handler_sort',
    ),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'debugger_requests',
      'relationship field' => 'rid',
      'label' => t('Debugger: Request ID'),
    ),
  );
  $data['debugger_traces']['time_delta'] = array(
    'title' => t('Time Delta'),
    'help'  => t('The execution time of that trace.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by time of the trace'),
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );
  $data['debugger_traces']['mem_delta'] = array(
    'title' => t('Memory delta'),
    'help'  => t('Memory used by the trace.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help'    => t('Filter by memory of the trace'),
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

  /* TODO: args, options */

  return $data;
}

